#include <msp/fs/dir.h>
#include <msp/strings/format.h>
#include "androidcxxcompiler.h"
#include "androidtools.h"
#include "architecture.h"
#include "builder.h"

using namespace std;
using namespace Msp;

AndroidCxxCompiler::AndroidCxxCompiler(Builder &b, const Architecture &a, const AndroidNdk &n):
	GnuCxxCompiler(b, a, n.get_platform_sysroot()),
	ndk(n)
{
	set_command((ndk.get_bin_dir()/command).str());
	build_info.incpath.insert(build_info.incpath.end(), system_path.begin(), system_path.end());
	build_info.libs.push_back("gnustl_static");
}

void AndroidCxxCompiler::do_prepare()
{
	GnuCxxCompiler::do_prepare();
	if(executable)
	{
		FS::Path libstdcxx_path = ndk.get_root_dir()/"sources"/"cxx-stl"/"gnu-libstdc++"/version;

		FS::Path public_dir = libstdcxx_path/"include";
		system_path.push_back(public_dir);
		build_info.incpath.push_back(public_dir);

		FS::Path arch_path = libstdcxx_path/"libs";
		builder.get_logger().log("files", format("Traversing %s", arch_path.str()));
		string arch_dir = architecture->best_match(list_files(arch_path));
		if(!arch_dir.empty())
		{
			build_info.incpath.push_back(libstdcxx_path/"libs"/arch_dir/"include");
			build_info.libpath.push_back(libstdcxx_path/"libs"/arch_dir);
		}
	}
}
